@@ -1,6 +1,6 @@
 (def jetty-version "9.2.10.v20150310")
 
-(defproject puppetlabs/trapperkeeper-webserver-jetty9 "1.5.11"
+(defproject puppetlabs/trapperkeeper-webserver-jetty9 "1.8.1"
   :description "A jetty9-based webserver implementation for use with the puppetlabs/trapperkeeper service framework."
   :url "https://github.com/puppetlabs/trapperkeeper-webserver-jetty9"
   :license {:name "Apache License, Version 2.0"
@@ -8,7 +8,7 @@
 
   :min-lein-version "2.7.1"
 
-  :parent-project {:coords [puppetlabs/clj-parent "0.1.3"]
+  :parent-project {:coords [puppetlabs/clj-parent "0.6.1"]
                    :inherit [:managed-dependencies]}
 
   ;; Abort when version ranges or version conflicts are detected in
@@ -39,12 +39,15 @@
                  [puppetlabs/ssl-utils]
                  [puppetlabs/kitchensink]
                  [puppetlabs/trapperkeeper]
+                 [puppetlabs/i18n]
+                 [puppetlabs/trapperkeeper-filesystem-watcher "1.1.0"]
                  ]
 
   :source-paths  ["src"]
   :java-source-paths  ["java"]
 
-  :plugins [[lein-parent "0.3.1"]]
+  :plugins [[lein-parent "0.3.1"]
+            [puppetlabs/i18n "0.8.0"]]
 
   :deploy-repositories [["releases" {:url "https://clojars.org/repo"
                                      :username :env/clojars_jenkins_username
