@@ -19,7 +19,7 @@
             [liberator.representation :as representation]
             [ring.util.request :as request]
             [ring.util.response :as rr]
-            [puppetlabs.i18n.core :as i18n :refer [tru]]))
+            [puppetlabs.i18n.core :as i18n]))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Constants
@@ -34,14 +34,14 @@
   [subject {:keys [cacert signeddir]}]
   (-> (if-let [certificate (ca/get-certificate subject cacert signeddir)]
         (rr/response certificate)
-        (rr/not-found (tru "Could not find certificate {0}" subject)))
+        (rr/not-found (i18n/tru "Could not find certificate {0}" subject)))
       (rr/content-type "text/plain")))
 
 (defn handle-get-certificate-request
   [subject {:keys [csrdir]}]
   (-> (if-let [certificate-request (ca/get-certificate-request subject csrdir)]
         (rr/response certificate-request)
-        (rr/not-found (tru "Could not find certificate_request {0}" subject)))
+        (rr/not-found (i18n/tru "Could not find certificate_request {0}" subject)))
       (rr/content-type "text/plain")))
 
 (schema/defn handle-put-certificate-request!
@@ -161,13 +161,13 @@
         :revoked
         ;; A signed cert must exist if we are to revoke it.
         (when-not (ca/certificate-exists? settings subject)
-          (conflict (tru "Cannot revoke certificate for host {0} without a signed certificate" subject)))
+          (conflict (i18n/tru "Cannot revoke certificate for host {0} without a signed certificate" subject)))
 
         :signed
         (or
           ;; A CSR must exist if we are to sign it.
           (when-not (ca/csr-exists? settings subject)
-            (conflict (tru "Cannot sign certificate for host {0} without a certificate request" subject)))
+            (conflict (i18n/tru "Cannot sign certificate for host {0} without a certificate request" subject)))
 
           ;; And the CSR must be valid.
           (when-let [error-message (ca/validate-csr settings subject)]
@@ -204,7 +204,7 @@
       ; https://github.com/clojure-liberator/liberator/pull/120
       ; ... but in our case, a 404 definitely makes more sense.
       (-> (assoc context :status 404)
-          (as-plain-text-response (tru "Invalid certificate subject.")))))
+          (as-plain-text-response (i18n/tru "Invalid certificate subject.")))))
 
   :handle-ok
   (fn [context]
@@ -219,20 +219,20 @@
           (if-let [desired-state (keyword (:desired_state json-body))]
             (if (schema/check ca/DesiredCertificateState desired-state)
               (malformed
-                (tru "State {0} invalid; Must specify desired state of ''signed'' or ''revoked'' for host {1}."
+                (i18n/tru "State {0} invalid; Must specify desired state of ''signed'' or ''revoked'' for host {1}."
                   (name desired-state) subject))
               ; this is the happy path. we have a body, it's parsable json,
               ; and the desired_state field is one of (signed revoked)
               [false {::json-body json-body}])
-            (malformed (tru "Missing required parameter \"desired_state\"")))
-          (malformed (tru "Request body is not JSON.")))
-        (malformed (tru "Empty request body.")))))
+            (malformed (i18n/tru "Missing required parameter \"desired_state\"")))
+          (malformed (i18n/tru "Request body is not JSON.")))
+        (malformed (i18n/tru "Empty request body.")))))
 
   :handle-malformed
   (fn [context]
     (if-let [message (::malformed context)]
       message
-      (tru "Bad Request.")))
+      (i18n/tru "Bad Request.")))
 
   :known-content-type?
   (fn [context]
