@@ -3,7 +3,7 @@
                                      HttpConfiguration HttpConnectionFactory
                                      ConnectionFactory AbstractConnectionFactory)
            (org.eclipse.jetty.server.handler AbstractHandler ContextHandler HandlerCollection
-                                             ContextHandlerCollection AllowSymLinkAliasChecker StatisticsHandler)
+                                             ContextHandlerCollection AllowSymLinkAliasChecker StatisticsHandler HandlerWrapper)
            (org.eclipse.jetty.util.resource Resource)
            (org.eclipse.jetty.util.thread QueuedThreadPool)
            (org.eclipse.jetty.util.ssl SslContextFactory)
@@ -23,7 +23,8 @@
            (java.lang.management ManagementFactory)
            (org.eclipse.jetty.jmx MBeanContainer)
            (org.eclipse.jetty.util URIUtil BlockingArrayQueue)
-           (java.io IOException))
+           (java.io IOException File)
+           (com.puppetlabs.trapperkeeper.services.webserver.jetty9.utils InternalSslContextFactory))
 
   (:require [ring.util.servlet :as servlet]
             [ring.util.codec :as codec]
@@ -31,7 +32,13 @@
             [clojure.tools.logging :as log]
             [puppetlabs.trapperkeeper.services.webserver.jetty9-config :as config]
             [puppetlabs.trapperkeeper.services.webserver.experimental.jetty9-websockets :as websockets]
-            [schema.core :as schema]))
+            [puppetlabs.trapperkeeper.services.webserver.normalized-uri-helpers
+             :as normalized-uri-helpers]
+            [puppetlabs.trapperkeeper.services.protocols.filesystem-watch-service
+             :as watch-protocol]
+            [schema.core :as schema]
+            [puppetlabs.i18n.core :as i18n]
+            [me.raynes.fs :as fs]))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; JDK SecurityProvider Hack
@@ -46,11 +53,11 @@
     (let [klass     (Class/forName "sun.security.pkcs11.SunPKCS11")
           blacklist (filter #(instance? klass %) (Security/getProviders))]
       (doseq [provider blacklist]
-        (log/info (str "Removing buggy security provider " provider))
+        (log/info (i18n/trs "Removing buggy security provider {0}" provider))
         (Security/removeProvider (.getName provider))))
     (catch ClassNotFoundException e)
     (catch Throwable e
-      (log/error e "Could not remove security providers; HTTPS may not work!"))))
+      (log/error e (i18n/trs "Could not remove security providers; HTTPS may not work!")))))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Schemas
@@ -62,7 +69,8 @@
 
 (def CommonOptions
   {(schema/optional-key :server-id) schema/Keyword
-   (schema/optional-key :redirect-if-no-trailing-slash) schema/Bool})
+   (schema/optional-key :redirect-if-no-trailing-slash) schema/Bool
+   (schema/optional-key :normalize-request-uri) schema/Bool})
 
 (def ContextHandlerOptions
   (assoc CommonOptions (schema/optional-key :context-listeners) [ServletContextListener]
@@ -133,7 +141,7 @@
    :overrides-read-by-webserver schema/Bool
    :overrides (schema/maybe {schema/Keyword schema/Any})
    :endpoints RegisteredEndpoints
-   :ssl-context-factory (schema/maybe SslContextFactory)})
+   :ssl-context-factory (schema/maybe InternalSslContextFactory)})
 
 (def ServerContext
   {:state     (schema/atom ServerContextState)
@@ -170,7 +178,7 @@
                                      :overrides-read-by-webserver
                                      true))]
     (doseq [key (keys overrides)]
-      (log/info (str "webserver config overridden for key '" (name key) "'")))
+      (log/info (i18n/trs "webserver config overridden for key ''{0}''" (name key))))
     (merge options overrides)))
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -182,11 +190,9 @@
   [{:keys [keystore-config client-auth ssl-crl-path cipher-suites protocols]}
    :- config/WebserverSslContextFactory]
   (if (some #(= "sslv3" %) (map str/lower-case protocols))
-    (log/warn (str "`ssl-protocols` contains SSLv3, a protocol with known "
-                  "vulnerabilities; we recommend removing it from the "
-                  "`ssl-protocols` list")))
+    (log/warn (i18n/trs "`ssl-protocols` contains SSLv3, a protocol with known vulnerabilities; we recommend removing it from the `ssl-protocols` list")))
 
-  (let [context (doto (SslContextFactory.)
+  (let [context (doto (InternalSslContextFactory.)
                   (.setKeyStore (:keystore keystore-config))
                   (.setKeyStorePassword (:key-password keystore-config))
                   (.setTrustStore (:truststore keystore-config))
@@ -398,7 +404,7 @@
   add-handler :- ContextHandler
   [webserver-context :- ServerContext
    handler :- ContextHandler
-   enable-trailing-slash-redirect?]
+   enable-trailing-slash-redirect? :- schema/Bool]
   (.setAllowNullPathInfo handler (not enable-trailing-slash-redirect?))
   (.addHandler (:handlers webserver-context) handler)
   ;; If this handler is being added after the server has been started, we
@@ -494,14 +500,14 @@
         (do
           (let [request-id    (.getRequestId this req)
                 async-context (.getAsyncContext req)]
-            (log/debug failure (str request-id " proxying failed"))
+            (log/debug failure (i18n/trs "{0} proxying failed" request-id))
             (if (.isCommitted resp)
               (try
                 (.sendError resp -1)
                 (.complete async-context)
                 (catch IOException _
-                  (log/debug failure (str request-id
-                                          " could not close the connection"))))
+                  (log/debug failure (i18n/trs "{0} could not close the connection"
+                                               request-id))))
               (do
                 (.resetBuffer resp)
                 (if (instance? TimeoutException failure)
@@ -523,6 +529,28 @@
                                                    [endpoint-map]
                                                    (conj % endpoint-map))))
 
+(schema/defn ^:always-validate max-request-body-size-handler*
+  [handler :- Handler
+   max-size :- schema/Int]
+  (proxy [HandlerWrapper] []
+    (handle [target ^Request base-request request response]
+      (let [request-size (.getContentLength base-request)]
+        (if (> request-size max-size)
+          (do
+            (.setStatus response HttpServletResponse/SC_REQUEST_ENTITY_TOO_LARGE)
+            (.setHandled base-request true))
+          (.handle handler target base-request request response))))))
+
+(schema/defn ^:always-validate max-request-body-size-handler
+  "Wrap a max-request-body-size handler around the supplied handler.  The
+  handler returns a 413 (request entity too large) error if the Content-Length
+  HTTP header on the incoming request exceeds the value specified as the
+  max-size parameter."
+  [handler :- Handler
+   max-size :- schema/Int]
+  (doto (max-request-body-size-handler* handler max-size)
+    (.setHandler handler)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Public
 
@@ -554,24 +582,23 @@
                                      :overrides-read-by-webserver
                                      true))]
     (doseq [key (keys overrides)]
-      (log/info (str "webserver config overridden for key '" (name key) "'")))
+      (log/info (i18n/trs "webserver config overridden for key ''{0}''" (name key))))
     (merge options overrides)))
 
 (schema/defn ^:always-validate shutdown
   [{:keys [server] :as webserver-context} :- ServerContext]
   (when-let [mbean-container (:mbean-container @(:state webserver-context))]
-    (log/debug "Cleaning up JMX MBean container")
-    (.destroy mbean-container))
+    (log/debug (i18n/trs "Cleaning up JMX MBean container"))
+    (.destroy mbean-container)
+    (swap! (:state webserver-context) assoc :mbean-container nil))
   (when (started? webserver-context)
-    (log/info "Shutting down web server.")
+    (log/info (i18n/trs "Shutting down web server."))
     (try
       (.stop server)
       (catch TimeoutException e
-        (log/errorf e
-                    (str "Web server failed to shut down gracefully in configured "
-                         "timeout period (%s); cancelling remaining requests.")
-                    (.getStopTimeout server))))
-    (log/info "Web server shutdown")))
+        (log/error e (i18n/trs "Web server failed to shut down gracefully in configured timeout period ({0}); cancelling remaining requests."
+                               (.getStopTimeout server)))))
+    (log/info (i18n/trs "Web server shutdown"))))
 
 (schema/defn ^:always-validate
   create-webserver :- ServerContext
@@ -625,13 +652,18 @@
     (.setHandlers hc (into-array Handler [(:handlers webserver-context)]))
     (let [shutdown-timeout (when (not (nil? (:shutdown-timeout-seconds options)))
                              (* 1000 (:shutdown-timeout-seconds options)))
-          maybe-zipped (if (or (not (contains? options :gzip-enable))
-                               (:gzip-enable options))
+          maybe-zipped (if (:gzip-enable options true)
                          (gzip-handler hc)
                          hc)
+          maybe-size-restricted (if-let [max-size (:request-body-max-size
+                                                   options)]
+                                  (max-request-body-size-handler
+                                   maybe-zipped
+                                   max-size)
+                                  maybe-zipped)
           maybe-logged (if log-handler
-                         (doto log-handler (.setHandler hc))
-                         maybe-zipped)
+                         (doto log-handler (.setHandler maybe-size-restricted))
+                         maybe-size-restricted)
           statistics-handler (if (or (nil? shutdown-timeout) (pos? shutdown-timeout))
                                (doto (StatisticsHandler.)
                                  (.setHandler maybe-logged))
@@ -649,13 +681,11 @@
   [webserver-context :- ServerContext
    config :- config/WebserverRawConfig]
   (let [webserver-context (create-webserver webserver-context config)]
-    (log/info "Starting web server.")
+    (log/info (i18n/trs "Starting web server."))
     (try
       (.start (:server webserver-context))
       (catch Exception e
-        (log/error
-          e
-          "Encountered error starting web server, so shutting down")
+        (log/error e (i18n/trs "Encountered error starting web server, so shutting down"))
         (shutdown webserver-context)
         (throw e)))
     webserver-context))
@@ -673,7 +703,8 @@
     options]
    (let [handler (ServletContextHandler. nil context-path ServletContextHandler/NO_SESSIONS)
          follow-links? (:follow-links? options)
-         enable-trailing-slash-redirect? (:enable-trailing-slash-redirect? options)]
+         enable-trailing-slash-redirect? (:enable-trailing-slash-redirect? options)
+         normalize-request-uri? (:normalize-request-uri? options)]
      (.setBaseResource handler (Resource/newResource base-path))
      (when follow-links?
        (.addAliasCheck handler (AllowSymLinkAliasChecker.)))
@@ -681,6 +712,9 @@
      (when-not (nil? context-listeners)
        (dorun (map #(.addEventListener handler %) context-listeners)))
      (.addServlet handler (ServletHolder. (DefaultServlet.)) "/")
+     (when normalize-request-uri?
+       (normalized-uri-helpers/add-normalized-uri-filter-to-servlet-handler!
+        handler))
      (add-handler webserver-context handler enable-trailing-slash-redirect?))))
 
 (schema/defn ^:always-validate
@@ -688,9 +722,14 @@
   [webserver-context :- ServerContext
    handler :- (schema/pred ifn? 'ifn?)
    path :- schema/Str
-   enable-trailing-slash-redirect?]
-  (let [ctxt-handler (doto (ContextHandler. path)
-                       (.setHandler (ring-handler handler)))]
+   enable-trailing-slash-redirect? :- schema/Bool
+   normalize-request-uri? :- schema/Bool]
+  (let [handler
+        (normalized-uri-helpers/handler-maybe-wrapped-with-normalized-uri
+         (ring-handler handler)
+         normalize-request-uri?)
+        ctxt-handler (doto (ContextHandler. path)
+                       (.setHandler handler))]
     (add-handler webserver-context ctxt-handler enable-trailing-slash-redirect?)))
 
 (schema/defn ^:always-validate
@@ -698,9 +737,14 @@
   [webserver-context :- ServerContext
    handlers :- websockets/WebsocketHandlers
    path :- schema/Str
-   enable-trailing-slash-redirect?]
-  (let [ctxt-handler (doto (ContextHandler. path)
-                       (.setHandler (websockets/websocket-handler handlers)))]
+   enable-trailing-slash-redirect? :- schema/Bool
+   normalize-request-uri? :- schema/Bool]
+  (let [handler
+        (normalized-uri-helpers/handler-maybe-wrapped-with-normalized-uri
+         (websockets/websocket-handler handlers)
+         normalize-request-uri?)
+        ctxt-handler (doto (ContextHandler. path)
+                       (.setHandler handler))]
     (add-handler webserver-context ctxt-handler enable-trailing-slash-redirect?)))
 
 (schema/defn ^:always-validate
@@ -709,12 +753,16 @@
    servlet :- Servlet
    path :- schema/Str
    servlet-init-params :- {schema/Any schema/Any}
-   enable-trailing-slash-redirect?]
+   enable-trailing-slash-redirect? :- schema/Bool
+   normalize-request-uri? :- schema/Bool]
   (let [holder (doto (ServletHolder. servlet)
                  (.setInitParameters servlet-init-params))
         handler (doto (ServletContextHandler. ServletContextHandler/SESSIONS)
                   (.setContextPath path)
                   (.addServlet holder "/*"))]
+    (when normalize-request-uri?
+      (normalized-uri-helpers/add-normalized-uri-filter-to-servlet-handler!
+       handler))
     (add-handler webserver-context handler enable-trailing-slash-redirect?)))
 
 (schema/defn ^:always-validate
@@ -725,10 +773,14 @@
   [webserver-context :- ServerContext
    war :- schema/Str
    path :- schema/Str
-   disable-redirects-no-slash?]
+   disable-redirects-no-slash? :- schema/Bool
+   normalize-request-uri? :- schema/Bool]
   (let [handler (doto (WebAppContext.)
                   (.setContextPath path)
                   (.setWar war))]
+    (when normalize-request-uri?
+      (normalized-uri-helpers/add-normalized-uri-filter-to-servlet-handler!
+       handler))
     (add-handler webserver-context handler disable-redirects-no-slash?)))
 
 (schema/defn ^:always-validate
@@ -750,13 +802,19 @@
    target :- ProxyTarget
    path :- schema/Str
    options :- ProxyOptions
-   disable-redirects-no-slash?]
+   disable-redirects-no-slash? :- schema/Bool]
   (let [target (update-in target [:path] remove-leading-slash)]
+    ;; This call hardcodes a value of 'false' for the 'normalize-request-uri?'
+    ;; parameter because the ProxyServlet already has its own logic for
+    ;; normalizing request URIs as it proxies them through.  Applying an
+    ;; extra layer of normalization in front of the ProxyServlet might otherwise
+    ;; cause requests to be proxied to an unintended URI.
     (add-servlet-handler webserver-context
                          (proxy-servlet webserver-context target options)
                          path
                          {}
-                         disable-redirects-no-slash?)))
+                         disable-redirects-no-slash?
+                         false)))
 
 (schema/defn ^:always-validate
    get-registered-endpoints :- RegisteredEndpoints
@@ -846,20 +904,16 @@
                (if (nil? (:overrides %))
                  (throw
                    (IllegalStateException.
-                     (str "overrides cannot be set because webserver has "
-                          "already processed the config")))
+                     (i18n/trs "overrides cannot be set because webserver has already processed the config")))
                  (throw
                    (IllegalStateException.
-                     (str "overrides cannot be set because they have "
-                          "already been set and webserver has already "
-                          "processed the config"))))
+                     (i18n/trs "overrides cannot be set because they have already been set and webserver has already processed the config"))))
              (nil? (:overrides %))
                (assoc % :overrides overrides)
              :else
                (throw
                  (IllegalStateException.
-                   (str "overrides cannot be set because they have "
-                        "already been set")))))))
+                   (i18n/trs "overrides cannot be set because they have already been set")))))))
 
 (schema/defn ^:always-validate join
   [webserver-context :- ServerContext]
@@ -889,8 +943,7 @@
                     server-id)]
     (when-not server-id
       (throw (IllegalArgumentException.
-               (str "no server-id was specified for this operation and "
-                    "no default server was specified in the configuration"))))
+               (i18n/trs "no server-id was specified for this operation and no default server was specified in the configuration"))))
     (server-id (:jetty9-servers service-context))))
 
 (defn get-default-server-from-config
@@ -903,6 +956,24 @@
                                             [server-id (initialize-context)]))
                  :default-server (get-default-server-from-config config)))
 
+(schema/defn ^:always-validate reload-crl-on-change!
+  "Reload the CRL file used by the supplied ssl-context-factory whenever any
+  changes to the file occur."
+  [ssl-context-factory :- InternalSslContextFactory
+   watcher :- (schema/protocol watch-protocol/Watcher)]
+  (when-let [crl-path (.getCrlPath ssl-context-factory)]
+    (let [normalized-crl-path (.getCanonicalPath (fs/file crl-path))]
+      (watch-protocol/add-watch-dir! watcher (fs/parent crl-path))
+      (watch-protocol/add-callback!
+       watcher
+       (fn [events]
+         (when (some #(and
+                       (:changed-path %)
+                       (= (.getCanonicalPath (:changed-path %))
+                          normalized-crl-path))
+                     events)
+           (.reload ssl-context-factory)))))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; Service Function Implementations
 
@@ -917,12 +988,14 @@
         endpoint-map      {:type              :context
                            :base-path         base-path
                            :context-listeners context-listeners}
-        enable-redirect  (:redirect-if-no-trailing-slash options)]
+        enable-redirect  (get options :redirect-if-no-trailing-slash false)
+        normalize-request-uri (get options :normalize-request-uri false)]
     (register-endpoint! state endpoint-map context-path)
     (add-context-handler s base-path context-path
                          context-listeners
                          {:follow-links?                   (:follow-links options)
-                          :enable-trailing-slash-redirect? enable-redirect})))
+                          :enable-trailing-slash-redirect? enable-redirect
+                          :normalize-request-uri? normalize-request-uri})))
 
 (schema/defn ^:always-validate init!
   [context config :- config/WebserverServiceRawConfig]
@@ -960,9 +1033,10 @@
         state         (:state s)
         endpoint-map  {:type     :ring}
 
-        enable-redirect  (:redirect-if-no-trailing-slash options)]
+        enable-redirect  (get options :redirect-if-no-trailing-slash false)
+        normalize-request-uri (get options :normalize-request-uri false)]
     (register-endpoint! state endpoint-map path)
-    (add-ring-handler s handler path enable-redirect)))
+    (add-ring-handler s handler path enable-redirect normalize-request-uri)))
 
 (schema/defn ^:always-validate add-websocket-handler!
   [context
@@ -973,9 +1047,14 @@
         s             (get-server-context context server-id)
         state         (:state s)
         endpoint-map  {:type     :websocket}
-        enable-redirect  (:redirect-if-no-trailing-slash options)]
+        enable-redirect  (get options :redirect-if-no-trailing-slash false)
+        normalize-request-uri (get options :normalize-request-uri false)]
     (register-endpoint! state endpoint-map path)
-    (add-websocket-handler s handlers path enable-redirect)))
+    (add-websocket-handler s
+                           handlers
+                           path
+                           enable-redirect
+                           normalize-request-uri)))
 
 (schema/defn ^:always-validate add-servlet-handler!
   [context servlet path options :- ServletHandlerOptions]
@@ -988,9 +1067,15 @@
         endpoint-map        {:type     :servlet
                              :servlet  (type servlet)}
 
-        enable-redirect  (:redirect-if-no-trailing-slash options)]
+        enable-redirect  (get options :redirect-if-no-trailing-slash false)
+        normalize-request-uri (get options :normalize-request-uri false)]
     (register-endpoint! state endpoint-map path)
-    (add-servlet-handler s servlet path servlet-init-params enable-redirect)))
+    (add-servlet-handler s
+                         servlet
+                         path
+                         servlet-init-params
+                         enable-redirect
+                         normalize-request-uri)))
 
 (schema/defn ^:always-validate add-war-handler!
   [context war path options :- CommonOptions]
@@ -1000,9 +1085,10 @@
         endpoint-map  {:type     :war
                        :war-path war}
 
-        enable-redirect  (:redirect-if-no-trailing-slash options)]
+        enable-redirect  (get options :redirect-if-no-trailing-slash false)
+        normalize-request-uri (get options :normalize-request-uri false)]
     (register-endpoint! state endpoint-map path)
-    (add-war-handler s war path enable-redirect)))
+    (add-war-handler s war path enable-redirect normalize-request-uri)))
 
 (schema/defn ^:always-validate add-proxy-route!
   [context target path options]
@@ -1013,6 +1099,6 @@
                        :target-host (:host target)
                        :target-port (:port target)
                        :target-path (:path target)}
-        enable-redirect  (:redirect-if-no-trailing-slash options)]
+        enable-redirect  (get options :redirect-if-no-trailing-slash false)]
     (register-endpoint! state endpoint-map path)
     (add-proxy-route s target path options enable-redirect)))
